EXPORTS

ivl_branch_island
ivl_branch_terminal

ivl_design_const
ivl_design_consts
ivl_design_discipline
ivl_design_disciplines
ivl_design_delay_sel
ivl_design_flag
ivl_design_process
ivl_design_root
ivl_design_roots
ivl_design_time_precision

ivl_const_bits
ivl_const_delay
ivl_const_file
ivl_const_lineno
ivl_const_nex
ivl_const_real
ivl_const_scope
ivl_const_signed
ivl_const_type
ivl_const_width

ivl_discipline_domain
ivl_discipline_flow
ivl_discipline_name
ivl_discipline_potential

ivl_enum_bits
ivl_enum_file
ivl_enum_lineno
ivl_enum_name
ivl_enum_names
ivl_enum_signed
ivl_enum_type
ivl_enum_width

ivl_event_any
ivl_event_basename
ivl_event_edg
ivl_event_file
ivl_event_lineno
ivl_event_name
ivl_event_nany
ivl_event_nedg
ivl_event_neg
ivl_event_nneg
ivl_event_npos
ivl_event_pos
ivl_event_scope

ivl_expr_type
ivl_expr_bits
ivl_expr_branch
ivl_expr_def
ivl_expr_delay_val
ivl_expr_dvalue
ivl_expr_enumtype
ivl_expr_event
ivl_expr_file
ivl_expr_lineno
ivl_expr_name
ivl_expr_nature
ivl_expr_net_type
ivl_expr_opcode
ivl_expr_oper1
ivl_expr_oper2
ivl_expr_oper3
ivl_expr_parameter
ivl_expr_parm
ivl_expr_parms
ivl_expr_property_idx
ivl_expr_repeat
ivl_expr_scope
ivl_expr_sel_type
ivl_expr_signal
ivl_expr_signed
ivl_expr_sized
ivl_expr_string
ivl_expr_uvalue
ivl_expr_value
ivl_expr_width

ivl_file_table_index
ivl_file_table_item
ivl_file_table_size

ivl_island_flag_set
ivl_island_flag_test

ivl_logic_attr
ivl_logic_attr_cnt
ivl_logic_attr_val
ivl_logic_basename
ivl_logic_delay
ivl_logic_drive0
ivl_logic_drive1
ivl_logic_file
ivl_logic_is_cassign
ivl_logic_lineno
ivl_logic_name
ivl_logic_pin
ivl_logic_pins
ivl_logic_scope
ivl_logic_type
ivl_logic_udp
ivl_logic_width

ivl_lpm_array
ivl_lpm_aset_value
ivl_lpm_async_clr
ivl_lpm_async_set
ivl_lpm_base
ivl_lpm_basename
ivl_lpm_clk
ivl_lpm_data
ivl_lpm_datab
ivl_lpm_define
ivl_lpm_delay
ivl_lpm_drive0
ivl_lpm_drive1
ivl_lpm_enable
ivl_lpm_file
ivl_lpm_lineno
ivl_lpm_name
ivl_lpm_negedge
ivl_lpm_q
ivl_lpm_scope
ivl_lpm_select
ivl_lpm_selects
ivl_lpm_signed
ivl_lpm_size
ivl_lpm_sset_value
ivl_lpm_string
ivl_lpm_sync_clr
ivl_lpm_sync_set
ivl_lpm_trigger
ivl_lpm_type
ivl_lpm_width

ivl_lval_idx
ivl_lval_mux
ivl_lval_nest
ivl_lval_part_off
ivl_lval_property_idx
ivl_lval_sel_type
ivl_lval_sig
ivl_lval_width

ivl_nature_name

ivl_nexus_get_private
ivl_nexus_name
ivl_nexus_ptrs
ivl_nexus_ptr
ivl_nexus_set_private

ivl_nexus_ptr_branch
ivl_nexus_ptr_con
ivl_nexus_ptr_drive0
ivl_nexus_ptr_drive1
ivl_nexus_ptr_pin
ivl_nexus_ptr_lpm
ivl_nexus_ptr_log
ivl_nexus_ptr_sig
ivl_nexus_ptr_switch

ivl_parameter_basename
ivl_parameter_expr
ivl_parameter_file
ivl_parameter_is_type
ivl_parameter_lineno
ivl_parameter_local
ivl_parameter_lsb
ivl_parameter_msb
ivl_parameter_scope
ivl_parameter_signed
ivl_parameter_width

ivl_path_condit
ivl_path_delay
ivl_path_is_condit
ivl_path_is_parallel
ivl_path_scope
ivl_path_source
ivl_path_source_negedge
ivl_path_source_posedge

ivl_process_analog
ivl_process_attr_cnt
ivl_process_attr_val
ivl_process_file
ivl_process_lineno
ivl_process_scope
ivl_process_stmt
ivl_process_type

ivl_scope_attr_cnt
ivl_scope_attr_val
ivl_scope_basename
ivl_scope_children
ivl_scope_child
ivl_scope_childs
ivl_scope_class
ivl_scope_classes
ivl_scope_def
ivl_scope_def_file
ivl_scope_def_lineno
ivl_scope_enumerate
ivl_scope_enumerates
ivl_scope_event
ivl_scope_events
ivl_scope_file
ivl_scope_func_type
ivl_scope_func_signed
ivl_scope_func_width
ivl_scope_is_auto
ivl_scope_is_cell
ivl_scope_lineno
ivl_scope_logs
ivl_scope_log
ivl_scope_lpms
ivl_scope_lpm
ivl_scope_mod_module_ports
ivl_scope_mod_module_port_name
ivl_scope_mod_module_port_type
ivl_scope_mod_module_port_width
ivl_scope_mod_port
ivl_scope_name
ivl_scope_param
ivl_scope_params
ivl_scope_parent
ivl_scope_port
ivl_scope_ports
ivl_scope_sigs
ivl_scope_sig
ivl_scope_switch
ivl_scope_switches
ivl_scope_time_precision
ivl_scope_time_units
ivl_scope_type
ivl_scope_tname

ivl_signal_array_addr_swapped
ivl_signal_array_base
ivl_signal_array_count
ivl_signal_attr
ivl_signal_attr_cnt
ivl_signal_attr_val
ivl_signal_basename
ivl_signal_data_type
ivl_signal_dimensions
ivl_signal_discipline
ivl_signal_file
ivl_signal_forced_net
ivl_signal_integer
ivl_signal_lineno
ivl_signal_local
ivl_signal_lsb
ivl_signal_msb
ivl_signal_name
ivl_signal_net_type
ivl_signal_nex
ivl_signal_npath
ivl_signal_packed_dimensions
ivl_signal_packed_lsb
ivl_signal_packed_msb
ivl_signal_path
ivl_signal_port
ivl_signal_scope
ivl_signal_signed
ivl_signal_type
ivl_signal_width

ivl_statement_type

ivl_stmt_block_count
ivl_stmt_block_scope
ivl_stmt_block_stmt
ivl_stmt_call
ivl_stmt_case_count
ivl_stmt_case_expr
ivl_stmt_case_quality
ivl_stmt_case_stmt
ivl_stmt_cond_expr
ivl_stmt_cond_false
ivl_stmt_cond_true
ivl_stmt_delay_expr
ivl_stmt_delay_val
ivl_stmt_events
ivl_stmt_file
ivl_stmt_flow_control
ivl_stmt_lexp
ivl_stmt_lineno
ivl_stmt_lval
ivl_stmt_lvals
ivl_stmt_lwidth
ivl_stmt_name
ivl_stmt_needs_t0_trigger
ivl_stmt_nevent
ivl_stmt_opcode
ivl_stmt_parm
ivl_stmt_parm_count
ivl_stmt_rval
ivl_stmt_sfunc_as_task
ivl_stmt_sub_stmt

ivl_switch_a
ivl_switch_b
ivl_switch_basename
ivl_switch_delay
ivl_switch_enable
ivl_switch_file
ivl_switch_island
ivl_switch_lineno
ivl_switch_offset
ivl_switch_part
ivl_switch_scope
ivl_switch_type
ivl_switch_width

ivl_type_base
ivl_type_element
ivl_type_name
ivl_type_packed_dimensions
ivl_type_packed_lsb
ivl_type_packed_msb
ivl_type_packed_width
ivl_type_prop_name
ivl_type_prop_type
ivl_type_properties
ivl_type_signed

ivl_udp_init
ivl_udp_file
ivl_udp_lineno
ivl_udp_name
ivl_udp_nin
ivl_udp_port
ivl_udp_row
ivl_udp_rows
ivl_udp_sequ
